# Generated by Django 3.2.9 on 2021-12-17 10:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0007_alter_collection_options'),
    ]

    operations = [
        migrations.RenameField(
            model_name='collectionitembase',
            old_name='item_type',
            new_name='type',
        ),
    ]
