# Generated by Django 4.0.1 on 2022-02-17 14:27

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0004_remove_fileitem_created_at_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='modified_at',
            field=models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='changed at'),
        ),
    ]
