# Generated by Django 4.0.2 on 2022-03-25 09:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0005_collection_modified_at'),
    ]

    operations = [
        migrations.AddField(
            model_name='svgitem',
            name='description',
            field=models.TextField(blank=True, help_text='This text will be used by screen readers, search engines, or when the image cannot be loaded', verbose_name='description'),
        ),
        migrations.AddField(
            model_name='svgitem',
            name='title',
            field=models.CharField(blank=True, help_text='The title is being used as a tooltip when the user hovers the mouse over the image', max_length=255, verbose_name='title'),
        ),
    ]
