<picture>
  <source type="image/webp"
          srcset="{{ item.admin_preview_webp_2x.url }} 2x, {{ item.admin_preview_webp.url }} 1x">
  <source srcset="{{ item.admin_preview_2x.url }} 2x, {{ item.admin_preview.url }} 1x">
  <img class="collection-item__image"
       src="{{ item.admin_preview.url }}"
       width="{{ width }}"
       height="{{ height }}"
       alt="{{ item.alt }}">
</picture>
